function output = fdr1d(dat, grp, nperm, nr, q, p0, vartype)
%Estimate for the 1-D local false discovery rate, t-statistic is used here.
% Matlab version of fdr1d in OCplus
% Arguments
%   dat, the matrix with row as links and column as samples.
%   grp, the group labels for the samples. For example, the dat is 4005 *
%       50 and the 1:25 are patients, 26:50 are controls, then grp can be
%       defined as [zeros(25,1);ones(25,1)];
%   nperm, the number for the permutation, default 100
%   nr, the number of bins for the region of statistics, default 100
%   q, the threshold for identifying significant links, default 0.2
%   p0, the prior proportion of null links.
%   vartype, two-sample t test with 'equal' or 'unequal' variance, default
%       'unequal'.
% Value
%   output, a struct,
%       .fdr, the local fdr for the links
%       .stat, the statistics for the links
%       .pvalue, the p-values for the links
%       .p0, the prior proportion of the null links used in the estimation
%       .th, the matrix for result with fdr < q.
%
%% Gan Xinjun, Fudan University, CCSB.
% xjgansdu@163.com, 2013.11.29
%

if (~exist('vartype','var') || isempty(vartype))
    vartype = 'N';   % default as N
end
if (~exist('nperm','var') || isempty(1000))
    nperm = 100;   % default as 2.4
end
if (~exist('nr','var') || isempty(nr))
    nr = 100;   % default as 2.4
end
if (~exist('q','var') || isempty(q))
    q = 0.2;   
end

%% check the dimension of data
[dx,nx] = size(dat); % nx, the number of samples in xdat 
lgx = length(grp);

if nx ~= lgx 
    error('The size of data and group are not consistent');
end

ob = ttest1d(dat, grp, vartype);
Z = ob.t;
pvalue = ob.p;


%% permutation procedure
zstar = zeros(dx * nperm, 1);
hw = waitbar(0,'Please wait, Permuting...');
for i = 1: nperm
    perm = randperm(nx);
    zperm = ttest1d(dat, grp(perm),vartype);
    zstar(((i - 1) * dx + 1):(i * dx)) = zperm.t;
    waitbar(i/nperm,hw)
end
delete(hw)
%% smoothing procedure
[f0fz, xbreaks] = smooth1d(Z, zstar, nr);

if (~exist('p0','var') || isempty(p0))
   p0 = 1/max(f0fz(abs(xbreaks) < 1));
end
fdr = min(p0 * f0fz,1);
output.fdr = interp1(xbreaks,fdr,Z);
output.stat = Z;
output.pvalue = pvalue;
sig_links = output.fdr < q;
vec = (1:dx)';
output.th = sortrows([Z(sig_links) pvalue(sig_links) output.fdr(sig_links) vec(sig_links)], 3);
output.p0 = p0;



