function output = fdr2d(xdat, grpx, zdat, grpz, nperm, nr, q,...
    p0, vartype, smoothp)
%The program for 2-D local false discovery rate, t-statistic is used here.
% Matlab version of fdr2d in OCplus
% Arguments
%   xdat, the first dataset, a matrix with row as links and column as samples.
%   grpx, the group labels for the samples in dataset 1. 
%   zdat, the second dataset, a matrix with row as links and column as samples.
%   grpz, the group labels for the samples in dataset 2.
%   nperm, the number for the permutation, default 100
%   nr, the number of bins for the region of each dimensional statistics, default 20
%   q, the threshold for identifying significant links, default 0.2
%   p0, the prior proportion of null links.
%   vartype, two-sample t test with 'equal' or 'unequal' variance, default
%       'unequal'.
%   smoothp, the smooth parameter in the smooth procedure.
% Value
%   output, a struct,
%       .fdr, the local fdr for the links
%       .fdr_contour, the fdr of the bins.
%       .stat, the statistics for the links
%       .pvalue, the p-values for the links
%       .xbreaks, the bins for the t-statistics of dataset 1
%       .ybreaks, the bins for the t-statistics of dataset 2
%       .p0, the prior proportion of the null links used in the estimation
%       .th, the matrix for result with fdr < q.
% c.f. Efron, Large scale inference, 2010. chapter 5.
%      Ploner, multidimensional false discovery rate, 2006.
%      R package: OCplus, locfdr, mfdr.
%
%% Gan Xinjun. Fudan University, CCSB. 
% ganxj@fudan.edu.cn
%

if (~exist('vartype','var') || isempty(vartype))
    vartype = 'N';   % default as N
end
if (~exist('nperm','var') || isempty(nperm))
    nperm = 100;   
end
if (~exist('nr','var') || isempty(nr))
    nr = 20;   
end
if (~exist('q','var') || isempty(q))
    q = 0.2;   
end

if (~exist('smoothp','var') || isempty(smoothp))
    smoothp = 0.2;   
end

% dx must be equal to dz, the number of links
[dx,nx] = size(xdat); % nx, the number of samples in xdat 
[dz,nz] = size(zdat); % nz, the number of samples in zdat
lgx = length(grpx);
lgz = length(grpz);

if nx ~= lgx | nz ~= lgz
    error('The size of data and group are not consistent');
end
if dx ~= dz
    error('The numbers of links in each datasets are not equal')
end

labelx = unique(grpx); % the label of group x
labelz = unique(grpz); % the label of group z
if length(labelx) ~= 2 | length(labelz) ~= 2
    error('The number groups of any dataset must be 2');
end

nx1 = sum(grpx == labelx(1)); % the size of patient in dataset 1
nx2 = nx - nx1;
nz1 = sum(grpz == labelz(1));
nz2 = nz - nz1;

if ((smoothp < 0.01) | (smoothp > 0.99)) 
    smoothp = min(max(smoothp, 0.01), 0.99)
    warning('smooth must be between 0.01 and 0.99 - reset to %d',smoothp)
end

%% calculate the observed statistics
ob = ttest2d(xdat, grpx, zdat, grpz, vartype);
Z = [ob.tx ob.tz];
pvalue = [ob.px ob.pz];


%% permutation procedure
zstar = zeros(dx * nperm, 2);

hw = waitbar(0,'Please wait, Permuting...');

for i = 1 : nperm
    % waitbar 
    permx = randperm(nx);
    % if the size between zdat and xdat equal use the same permutation
    if nx1 == nz1 & nx2 == nz2
        permz = permx;
    else
        permz = randperm(nz);
    end
    grp1 = grpx(permx);
    grp2 = grpz(permz);
    
    zperm = ttest2d(xdat, grp1, zdat, grp2, vartype);
    zstar(((i - 1) * dx + 1):(i * dx),:) = [zperm.tx zperm.tz];
    
    waitbar(i/nperm,hw)
end

delete(hw);


%% smoothing procedure
[b, xbreaks, ybreaks]= smooth2d(Z, zstar, nr, smoothp);

f0fz = b./((1 - b) * nperm);

%% estimate p0
if (~exist('p0','var') || isempty(p0)) 
    %zndx = findInterval(0,xbreaks)
    [ecdfv, xv] = ecdf(xbreaks);
    [~, ecdf_loc] = histc(0,unique([-inf;xv;inf]));
    zndx = length(xbreaks) * ecdfv(ecdf_loc);
    nn = length(ybreaks);
    mndx = (floor(nn/4):ceil(3 * nn/4))';
    p0 = min(1./f0fz(zndx, mndx));
end
if (p0 > 1) 
    warning('p0 estimate > 1. This may be due to oversmoothing')
end
fdr = min(p0 * f0fz, 1);

%% monotone procedure
fdr = DoConstrain(fdr, xbreaks);
output.fdr = interp2(xbreaks,ybreaks,fdr,Z(:,1),Z(:,2),'linear'); 
output.fdr_contour = fdr;
output.stat = Z;
output.pvalue = pvalue;
output.xbreaks = xbreaks;
output.ybreaks = ybreaks;
sig_links = output.fdr < q;
vec = (1:dx)';
output.th = sortrows([Z(sig_links,:) pvalue(sig_links,:) output.fdr(sig_links) vec(sig_links)],5);






function fdr = DoConstrain(fdr, xbreaks)
%%matlab version DoConstrain in OCplus
rhs = xbreaks >= 0;
lhs = xbreaks < 0;
cfdr = size(fdr,2);
for j = 1: cfdr
    
    aa = fdr(rhs, j);
    
    fdr(rhs, j) = cummin(aa);
    
    temp = fdr(lhs,j);
    
    bb = temp(end:-1:1);
    
    cc = cummin(bb);
    
    fdr(lhs,j) = cc(end:-1:1);

end




