function output = meta2d(xdat, grpx, zdat, grpz, nperm, nr, q,...
    p0, vartype, smoothp)
%The program for combination of the statistics, using meta-analysis and
% give the 1-D local fdr for the combination.
% Arguments
%   xdat, the first dataset, a matrix with row as links and column as samples.
%   grpx, the group labels for the samples in dataset 1. 
%   zdat, the second dataset, a matrix with row as links and column as samples.
%   grpz, the group labels for the samples in dataset 2.
%   nperm, the number for the permutation, default 100
%   nr, the number of bins for the region of each dimensional statistics, default 100
%   q, the threshold for identifying significant links, default 0.2
%   p0, the prior proportion of null links.
%   vartype, two-sample t test with 'equal' or 'unequal' variance, default
%       'unequal'.
%   smoothp, the smooth parameter in the smooth procedure.
% Value
%   output, a struct,
%       .fdr, the local fdr for the links
%       .fdr_contour, the fdr of the bins
%       .stat, the statistics for the links
%       .pvalue, the p-values for the links
%       .xbreaks, the bins for the t-statistics of dataset 1
%       .ybreaks, the bins for the t-statistics of dataset 2
%       .p0, the prior proportion of the null links used in the estimation
%       .th, the matrix for result with fdr < q.
%
%% Gan Xinjun, Fudan University, CCSB. 
% ganxj@fudan.edu.cn
%

if (~exist('vartype','var') || isempty(vartype))
    vartype = 'N';   % default as N
end
if (~exist('nperm','var') || isempty(nperm))
    nperm = 100;   
end
if (~exist('nr','var') || isempty(nr))
    nr = 100;   
end
if (~exist('q','var') || isempty(q))
    q = 0.2;   
end

if (~exist('smoothp','var') || isempty(smoothp))
    smoothp = 0.2;   
end

% dx must be equal to dz, the number of links
[dx,nx] = size(xdat); % nx, the number of samples in xdat 
[dz,nz] = size(zdat); % nz, the number of samples in zdat
lgx = length(grpx);
lgz = length(grpz);

if nx ~= lgx | nz ~= lgz
    error('The size of data and group are not consistent');
end
if dx ~= dz
    error('The numbers of links in each datasets are not equal')
end

labelx = unique(grpx); % the label of group x
labelz = unique(grpz); % the label of group z
if length(labelx) ~= 2 | length(labelz) ~= 2
    error('The number groups of any dataset must be 2');
end

nx1 = sum(grpx == labelx(1)); % the size of patient in dataset 1
nx2 = nx - nx1;
nz1 = sum(grpz == labelz(1));
nz2 = nz - nz1;

if ((smoothp < 0.01) | (smoothp > 0.99)) 
    smoothp = min(max(smoothp, 0.01), 0.99)
    warning('smooth must be between 0.01 and 0.99 - reset to %d',smoothp)
end

%% calculate the observed statistics
ob = ttest2d(xdat, grpx, zdat, grpz, vartype);

Z_n = [norminv(ob.px) norminv(ob.pz)]; % change to one-sided normal
% estimation for weight
hatdelta1 = sqrt(nx/(nx-2)).* Z_n(:,1);
sigma1squa = nx/(nx1*nx2) + hatdelta1.^2./(2*nx);

hatdelta2 = sqrt(nz/(nz-2)).*Z_n(:,2);
sigma2squa = nz/(nz1*nz2) + hatdelta2.^2/(2*nz);
omega1 = (1./sigma1squa)./((1./sigma1squa)+(1./sigma2squa));
omega2 = (1./sigma2squa)./((1./sigma1squa)+(1./sigma2squa));
Z = omega1.*Z_n(:,1) + omega2.*Z_n(:,2); 



pvalue = [ob.px ob.pz];


%% permutation procedure
zstar_n = zeros(dx * nperm, 2);
zstar = zeros(dx * nperm, 1);
hw = waitbar(0,'Please wait, Permuting...');

for i = 1 : nperm
    % waitbar 
    permx = randperm(nx);
    % if the size between zdat and xdat equal use the same permutation
    if nx1 == nz1 & nx2 == nz2
        permz = permx;
    else
        permz = randperm(nz);
    end
    grp1 = grpx(permx);
    grp2 = grpz(permz);
    
    zperm = ttest2d(xdat, grp1, zdat, grp2, vartype);
    % zstar(((i - 1) * dx + 1):(i * dx)) = -2*(log(zperm.px)+log(zperm.pz));
    zstar_n(((i - 1) * dx + 1):(i * dx),1) = norminv(zperm.px);
    zstar_n(((i - 1) * dx + 1):(i * dx),2) = norminv(zperm.pz);
         
    
    ndx = grp1 == labelx(1);
    ndz = grp2 == labelz(1);
    nx1 = sum(ndx);
    nx2 = nx - nx1;
    nz1 = sum(ndz);
    nz2 = nz - nz2;
    hatdelta1 = sqrt( nx / (nx - 2))*zstar_n(((i - 1) * dx + 1):(i * dx), 1);
    sigma1squa = nx/(nx1 * nx2) + hatdelta1.^2/(2*nx);
    hatdelta2 = sqrt( nz / (nz - 2))*zstar_n(((i - 1) * dx + 1):(i * dx), 2);
    sigma2squa = nz/(nz1*nz2) + hatdelta2.^2/(2 * nz);
    omega1 = (1./sigma1squa)./((1./sigma1squa)+(1./sigma2squa));
    omega2 = (1./sigma2squa)./((1./sigma1squa)+(1./sigma2squa));
    zstar(((i - 1) * dx + 1):(i * dx)) = omega1 .* (zstar_n(((i - 1) * dx + 1):(i * dx), 1)) +...
        omega2 .* (zstar_n(((i - 1) * dx + 1):(i * dx), 2));
    waitbar(i/nperm,hw)
end

delete(hw);
%% smoothing procedure
[f0fz, xbreaks] = smooth1d(Z, zstar, nr);

if (~exist('p0','var') || isempty(p0))
   p0 = 1/max(f0fz(abs(xbreaks) < 1));
end
fdr = min(p0 * f0fz,1);
output.fdr = interp1(xbreaks,fdr,Z);
output.stat = [ob.tx ob.tz];
output.pvalue = pvalue;
sig_links = output.fdr < q;
vec = (1:dx)';
output.th = sortrows([Z(sig_links) pvalue(sig_links) output.fdr(sig_links) vec(sig_links)],3);
output.p0 = p0;



