function output = sam(dat, grp, nperm, delta, threshold, vartype, fold)
%Matlab version of siggenes package in R
% provided by Eric NIMPAYE, UAE-ENSAT-MISC, 2013.on exchange files in the
% website of mathworks. With a few modifications to fit our COMMON package
% Arguments
%   dat, the matrix with row as links and column as samples.
%   grp, the group labels for the samples. For example, the dat is 4005 *
%       50 and the 1:25 are patients, 26:50 are controls, then grp can be
%       defined as [zeros(25,1);ones(25,1)];
%   nperm, the number for the permutation, default 100
%   delta, the parameter threshold for significant links, see samr package
%       for help, default 2.4
%   p0, the prior proportion of null links.
%   vartype, two-sample t test with 'equal' or 'unequal' variance, default
%       'unequal'.
%   fold, the fold change parameter, default as 1.
% 
% Value
%   output, a struct,
%       .fdr, the local fdr for the links
%       .stat, the statistics for the links
%       .pvalue, the p-values for the links
%       .dev, the deviation in stat (denominator of t-stat)
%       .s0, the modified parameter of deviation
%       .p0, the prior proportion of the null links used in the estimation
%       .th, the matrix for result with fdr < q.
%       .fulldelta, the full result for delta in (0:0.1:5);
%
%% Gan Xinjun, Fudan University, CCSB.
%

if (~exist('vartype','var') || isempty(vartype))
    vartype = 'N';   % default as N
end
if (~exist('fold','var') || isempty(fold))
    fold = 1;   % default as 1
end
if (~exist('nperm','var') || isempty(1000))
    nperm = 1000;   % default as 2.4
end
if (~exist('delta','var') || isempty(delta))
    delta = 2.4;   % default as 2.4
end



[dim,n] = size(dat);
lg = length(grp);
if abs(n-lg) > eps
    error('The size of data and group are not consistent');
end

%% fold changes procedure
dfold = foldchanges(dat,grp,fold);
% check the number of genes with fold changes
n_fulfill = sum (dfold (:, 2));
if n_fulfill == 0
    error ('fold:check', strcat ('None of the variables has a fold',...
        ' change larger than %g.\n\n', dfold));
end

if n_fulfill < 20
    error ('fold:check', strcat ('Less than 20 variables have a fold',...
        ' change larger than %g.\n\n', dfold));
end


fold_out = dfold (:, 1);

if n_fulfill < size(dfold, 1)
    exclud_fc = find(dfold (:, 2) == 0);

    fold_out(exclud_fc) = [];
    dat(exclud_fc, :) = [];
    warning ('fold:check', strcat ('The following genes are excluded',...
        ' on the basis of their low fold changes : \n'));
    fprintf ('\n\tRow\tName\n');
    for k = 1:length (exclud_fc)
        fprintf ('\t%i\n', exclud_fc(k));
    end
    fprintf ('\n');
end


%% get the statistics
[difference, se] = sam_tstat(dat,grp,vartype);
% Exclude links wich have null variance
if sum (se == 0) ~= 0
    var0_links = find (se == 0);   % the idx of var0_links
    difference(var0_links) = [];
    se(var0_links) = [];
    dat(var0_links, :) = [];

    if fold > 0
        fold_out (var0_links) = [];
    end

    warning ('variance:check', strcat ('The following genes are',...
        ' excluded on the basis of their low variances : \n'));

    fprintf ('\n\tRow\tName\n');
    for k = 1:length (var0_genes)
        fprintf ('\t%i\t%s\n', var0_genes (k), gene_names {var0_genes (k)});
    end
    fprintf ('\n');
    [b_nrow, b_ncol] = size(dat);
    fprintf (strcat ('New data dimensions :\n\tobservations count :',...
        ' %i\n\texperiments count : %i\n\n'), b_nrow, b_ncol);
end
%% get the modified coefficient s0
s0 = get_szero(difference, se);
stat_ob = difference./(se + s0);
%% permutation procedure
[stat_perm, p_value, vec_false] = samperm_proc(dat,grp,nperm,stat_ob,s0,vartype);
p0_pri = p0est(p_value);
q_value = qvaluecal(p0_pri,p_value);

%% analysis of the result
% full result
sam_full_fdr = sam_collect(stat_ob, stat_perm, p0_pri, vec_false);
% for the given delta
sam_fdr = sam_collect(stat_ob, stat_perm, p0_pri, vec_false, delta);
%greater than cutup, lower than cutlow
sig_inf = find(stat_ob <= sam_fdr (:, 6));
sig_pos = find(stat_ob >= sam_fdr (:, 7));

sig_links = [sig_inf;sig_pos]';
if ~exist('threshold','var')
    plotsam(stat_ob, stat_perm, sam_fdr, delta);
else
    sig_links = q_value < threshold;
end
vec = (1:dim)';
output.stat = stat_ob;
output.pvalue = p_value;
output.dev = se;
output.p0 = p0_pri;
output.s0 = s0;
output.fdr = q_value; % FDR not local fdr
output.fulldelta = sam_full_fdr;
output.th = [stat_ob(sig_links) se(sig_links) p_value(sig_links)...
    q_value(sig_links) fold_out(sig_links) vec(sig_links)];
% 




function dfold = foldchanges(dat,grp,fold)
%% computes the observations fold changes.
%
% dfold = foldchanges(dat,grp,fold) returns the fold changes dfold of the
% data divided into classes designed as indicated by grp.
%
% VALUE
%       dfold : the observations fold changes.
%% This is a modified version of Eric NIMPAYE, UAE-ENSAT-MISC, 2013.

label = unique (grp);

if fold < 1
    fold = 1 / fold;
end

x = dat (:, grp == label (1));
y = dat (:, grp == label (2));

mx = mean(x, 2);
my = mean(y, 2);

fold_out = 2 .^ (my - mx);

fulfill = zeros (1, length(fold_out));
fulfill (fold_out >= fold) = 1;
fulfill (fold_out <= (1 / fold)) = 1;

dfold = zeros (length (fold_out), 2);
dfold (:, 1) = fold_out;
dfold (:, 2) = fulfill;


function s0 = get_szero(difference, se, s_alpha)
%% FUDGE_FACTOR determines the fudge factor.
%
% s0 = get_szero(diference, se) returns the fudge factor s0 computed
% based on the numerator difference and the denominator se of t-test.
%
% VALUE
%       s0 : the computed fudge factor.

%% copy with a few modification from Eric NIMPAYE, UAE-ENSAT-MISC, 2013.

% s_alpha \in (0,1), linspace of [0,1]
if (~exist('s_alpha','var') || isempty(s_alpha))
    s_alpha = 0.05;   % default as 0.05
end
s_alpha = (0:s_alpha:1);

if length (s_alpha) == 1
    s0 = real (quantile(se, s_alpha));
    return;
end

s_quan = real(quantile (se, s_alpha));
n_alpha = length(s_quan);
s_len = length(se);

n_uni_se = length (unique(se));
d_mat = (difference * ones(1,n_alpha))./(se*ones(1,n_alpha) + ones(s_len,1) * s_quan);
if n_uni_se < 25
    error ('fudge:few_links', strcat ('For the computation of the fugde ',...
        'factor, there should be at least 25 links with differing standard',...
        ' deviations.\n'));
end

if n_uni_se > 500
    n_int = 101;
else
    n_int = floor (n_uni_se / 5);
end

% (to - from) / (length - 1)
buff_foot = 1 / (n_int - 1);
quan = real(quantile(se, 0:buff_foot:1));

%round to the 8th digit after the comma
quan = floor((quan * power (10, 8))) / power (10, 8);
quan = unique(quan);
n_int = length(quan);
% len_s = length(se);
s_sort = sort(se);
% int_s = zeros (1, len_s);
% for i = 1:len_s
%     j = 2;
%     while (s_sort (i) > quan (j)) && (j < n_int)
%         j = j + 1;
%     end   
%     int_s (i) = j - 1;
% end
num_rep = histc(s_sort,quan);
ind_rep = zeros(1,sum(num_rep));
ind_rep(cumsum([1;num_rep(1:end-1)])) = 1;
int_s = cumsum(ind_rep);
int_s(end) = n_int-1;

mad_mat = zeros(n_int - 1, size (d_mat, 2));


for i = 1:n_int - 1
    buff_ind = (int_s == i);
    mad_mat(i, :) = 1.4826 * mad(d_mat(buff_ind, :), 1);
end
mad_mat (isnan(mad_mat)) = 0;


vec_cv = std(mad_mat)./mean(mad_mat);
% [cv_min, which_min] = min(vec_cv);

s_quan = s_quan / 10;

buff_ind = s_quan == 0;
s_quan (buff_ind) = [];
vec_cv (buff_ind) = [];
[~, which_min] = min(vec_cv);
s0 = s_quan(which_min);


function p0 = p0est(p_value)
%% Estimation of the prior proportion of null links
% copy from Eric NIMPAYE, UAE-ENSAT-MISC, 2013.
lambda = (0:0.05:0.95); % used for the estimation of pi0.
le_la = length (lambda);
ncs_pred = max(lambda);
vec_p0 = zeros (1, le_la);
for i = 1:le_la
    vec_p0 (i) = mean (p_value (p_value > lambda (i))) ./ (1 - lambda (i));
end
lambda(isnan(vec_p0)) = [];
vec_p0(isnan(vec_p0)) = []; % delete nan
if le_la == 1
    p0 = min ([1, vec_p0]);
    return;
end

p0 = median (spline (lambda, vec_p0, ncs_pred), 1) / 100;

if p0 <= 0
    p0 = vec_p0 (lambda == 0.5);
end



function q_value = qvaluecal(p0,p_value)
%% calculation of the q_value, min(FDR).
p_na = 0;

if sum (isnan (p_value)) > 0 % delete nan
    p_na = find (isnan (p_value));
    p_value (isnan (p_value)) = [];
end

m = length (p_value);
p_sort = sort (p_value);
q_value = (p0 * m) .* p_sort ./ (1:m)';

q_value (m) = min (q_value(m), 1);

for i = (m-1):-1:1
    q_value (i) = min (q_value(i), q_value(i+1));
end

% buff_qord = ranking (p_value);
[~, buff_ranks] = sort (p_value);
[~, buff_qord] = sort (buff_ranks);

q_value (buff_qord) = q_value;

if p_na ~= 0
    q_value (p_na) = NaN;
end


function [mat_fdr, mat_fdr_hdr] = sam_collect(stat_ob, stat_perm, p0, vec_false, delta)
% * MAT_FDR = sam_collect (stat_ob,stat_perm,vec_false) determines the
%
%       - delta : limens from which expressed observations are picked up;
%
%       - prior probability that an observation is expressed;
%
%       - false called opbservations;
%
%       - FDR , the False Discovery Rate.
%
% All those values are stacked into MAT_FDR ; they are computed based on the
% D (observed) scores, D_BAR (expected) scores, the limens of falsely called
% observations VEC_FALSE, and the preferred values of DELTA to include in
% the findings.
%
% * [MAT_FDR, MAT_FDR_HDR] = sam_collect (stat_ob,stat_perm,vec_false) returns
% the header MAT_FDR_HDR of the statistics container MAT_FDR.
%
% VALUE
%       MAT_FDR : the differential expressed observations' metrics.
%
%       MAT_FDR_HDR : the metric container's header.

%% Copy from Eric NIMPAYE, UAE-ENSAT-MISC, 2013.
d_sort = sort (stat_ob);
d_bar_sort = sort (stat_perm);
d_diff = (d_sort - d_bar_sort)';
m = length (d_diff);
% get delta
if (~exist('delta','var') || isempty(delta))
    buff_absd = abs (d_diff);
    buff_inf = min (buff_absd);
    buff_sup = max (buff_absd);

    buff_inf = max (0.1, buff_inf);
    buff_sup = max (1, buff_sup);

    buff_step = (buff_sup - buff_inf) / (50 - 1);

    buff_seq = buff_inf:buff_step:buff_sup;

    delta = round (buff_seq * 10) / 10;
    
end

le_delta = length(delta);


buff_dmp = d_bar_sort (d_bar_sort >= 0);
if isempty (buff_dmp)
    error ('stat_cal:no_gene', 'No gene statistics for delta = %g.\n', delta);
end
buff_dpos = find (d_bar_sort == min (buff_dmp (1)));

j0 = buff_dpos (1);

mat_fdr_hdr = {'Delta', 'p0', 'False', 'Called', 'FDR', 'cutlow', 'cutup',...
    '-thresh', '+thresh'};

mat_fdr = zeros (le_delta, 9);

mat_fdr (:, 1) = delta;
mat_fdr (:, 2) = p0;

buff_flat = reshape (stat_ob, 1, size (stat_ob, 1) * size (stat_ob, 2));
[~, buff_ranks] = sort (buff_flat);
[~, buff_ord] = sort (buff_ranks);

vec_order = vec_false (buff_ord);
vec_order = vec_order (~ isnan (vec_order));

for i = 1:le_delta
    %j1
    if sum (d_diff (j0:m) >= delta (i)) ~= 0
        buff_pos = find (d_diff (j0:m) >= delta (i));
        j1 = j0 - 1 + min (buff_pos);
    else
        j1 = m + 1;
    end
    mat_fdr (i, 9) = j1;
    
    %cutup
    if j1 ~= m + 1
        mat_fdr (i, 7) = d_sort (j1);
    else
        mat_fdr (i, 7) = Inf;
    end
    
    %j2
    if (sum (d_diff (1 : (j0 - 1)) <= (- delta (i))) ~= 0) && (j0 ~= 1)
        buff_pos = find (d_diff (1 : (j0 - 1)) <= (- delta (i)));
        j2 = max (buff_pos);
    else
        j2 = 0;
    end
    mat_fdr (i, 8) = j2;
    
    %cutlow
    if j2 ~= 0
        mat_fdr (i, 6) = d_sort (j2);
    else
        mat_fdr (i, 6) = -Inf;
    end
    
    %Called
    mat_fdr (i, 4) = m - j1 + 1 + j2;
    
    %False
    if j1 == m + 1
        buff_add = 0;
    else
        buff_add = vec_order (j1);
    end
    if j2 == 0
        buff_add_2 = 0;
    else
        buff_add_2 = vec_order (j2);
    end
    mat_fdr (i, 3) = buff_add + buff_add_2;
    
    %FDR
    buff_denum = max (mat_fdr (i, 4), 1);
    buff_num = p0 * mat_fdr (i, 3);
    mat_fdr (i, 5) = buff_num / buff_denum;
end



